@@ -57,7 +57,7 @@ import java.util.jar.JarInputStream;
  * contained the annotation), but were unable to be loaded.
  *
  * @author David Blevins
- * @version $Rev: 661180 $ $Date: 2008-05-29 04:08:46 +0200 (Do, 29. Mai 2008) $
+ * @version $Rev: 731945 $ $Date: 2009-01-06 15:19:28 +0100 (Di, 06. Jan 2009) $
  */
 public class ClassFinder {
     private final Map<String, List<Info>> annotated = new HashMap<String, List<Info>>();
@@ -245,6 +245,49 @@ public class ClassFinder {
         return classes;
     }
 
+    /**
+     * Naive implementation - works extremelly slow O(n^3)
+     *
+     * @param annotation
+     * @return list of directly or indirectly (inherited) annotated classes
+     */
+    public List<Class> findInheritedAnnotatedClasses(Class<? extends Annotation> annotation) {
+        classesNotLoaded.clear();
+        List<Class> classes = new ArrayList<Class>();
+        List<Info> infos = getAnnotationInfos(annotation.getName());
+        for (Info info : infos) {
+            try {
+                classes.add(((ClassInfo) info).get());
+            } catch (ClassNotFoundException cnfe) {
+                // TODO: ignored, but a log message would be appropriate
+            }
+        }
+        boolean annClassFound;
+        List<ClassInfo> tempClassInfos = new ArrayList<ClassInfo>(classInfos);
+        do {
+            annClassFound = false;
+            for (int pos = 0; pos < tempClassInfos.size(); pos++) {
+                ClassInfo classInfo = tempClassInfos.get(pos);
+                try {
+                    String superType = classInfo.getSuperType();
+                    for (Class clazz : classes) {
+                        if (superType.equals(clazz.getName())) {
+                            classes.add(classInfo.get());
+                            tempClassInfos.remove(pos);
+                            annClassFound = true;
+                            break;
+                        }
+                    }
+                } catch (ClassNotFoundException e) {
+                    classesNotLoaded.add(classInfo.getName());
+                } catch (NoClassDefFoundError e) {
+                    classesNotLoaded.add(classInfo.getName());
+                }
+            }
+        } while (annClassFound);
+        return classes;
+    }
+
     public List<Method> findAnnotatedMethods(Class<? extends Annotation> annotation) {
         classesNotLoaded.clear();
         List<ClassInfo> seen = new ArrayList<ClassInfo>();
