@@ -16,10 +16,14 @@
  */
 package org.acme.foo;
 
+import java.lang.annotation.Inherited;
+import java.lang.annotation.Retention;
+import java.lang.annotation.Target;
+
 /**
- * @version $Revision: 469417 $ $Date: 2006-10-31 09:50:58 +0100 (Di, 31. Okt 2006) $
+ * @version $Revision: 729546 $ $Date: 2008-12-26 23:20:45 +0100 (Fr, 26. Dez 2008) $
  */
-@java.lang.annotation.Target(value = {java.lang.annotation.ElementType.TYPE})
-@java.lang.annotation.Retention(value = java.lang.annotation.RetentionPolicy.RUNTIME)
+@Target(value = {java.lang.annotation.ElementType.TYPE})
+@Retention(value = java.lang.annotation.RetentionPolicy.RUNTIME)
 public @interface Holiday {
 }
